# WARNING: Please don't edit this file. It was generated by Python/WinRT v2.0.0-beta.2

import winrt.system
from winrt import _winrt_windows_devices_pwm_provider

__all__ = [
    "IPwmControllerProvider",
    "IPwmProvider",
]

IPwmControllerProvider = _winrt_windows_devices_pwm_provider.IPwmControllerProvider
IPwmProvider = _winrt_windows_devices_pwm_provider.IPwmProvider
